# This file was auto-generated by Fern from our API Definition.

import json
import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from .core.api_error import ApiError
from .core.jsonable_encoder import jsonable_encoder
from .core.remove_none_from_headers import remove_none_from_headers
from .environment import ChimaEnvironment
from .types.converse_response import ConverseResponse
from .types.generate_text_response import GenerateTextResponse
from .types.search_response import SearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class Chima:
    def __init__(self, *, environment: ChimaEnvironment = ChimaEnvironment.PRODUCTION, token: str):
        self._environment = environment
        self._token = token

    def search(
        self,
        group_id: str,
        *,
        query: str,
        provider: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        pdf: typing.Optional[str] = OMIT,
    ) -> SearchResponse:
        _request: typing.Dict[str, typing.Any] = {"query": query}
        if provider is not OMIT:
            _request["provider"] = provider
        if id is not OMIT:
            _request["id"] = id
        if pdf is not OMIT:
            _request["pdf"] = pdf
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", f"search/{group_id}"),
            json=jsonable_encoder(_request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SearchResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def stream(self, *, level_context: str) -> typing.Iterator[str]:
        with httpx.stream(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "api_streaming"),
            json=jsonable_encoder({"level_context": level_context}),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        ) as _response:
            if 200 <= _response.status_code < 300:
                for _text in _response.iter_text():
                    if len(_text) == 0:
                        continue
                    yield pydantic.parse_obj_as(str, json.loads(_text))  # type: ignore
                return
            try:
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    def generate(
        self,
        *,
        gender: str,
        country: str,
        age_bin: str,
        bmi_bin: str,
        active_journey: str,
        need: str,
        equipment: str,
        day_type: str,
        cluster_workout: str,
        cluster_challenge: str,
        last_workout_completed: str,
        last_workout_week: str,
        last_week_completed: int,
        delta_moves: str,
        notification_type: str,
    ) -> GenerateTextResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "generate"),
            json=jsonable_encoder(
                {
                    "gender": gender,
                    "country": country,
                    "age_bin": age_bin,
                    "bmi_bin": bmi_bin,
                    "active_journey": active_journey,
                    "need": need,
                    "equipment": equipment,
                    "day_type": day_type,
                    "cluster_workout": cluster_workout,
                    "cluster_challenge": cluster_challenge,
                    "last_workout_completed": last_workout_completed,
                    "last_workout_week": last_workout_week,
                    "last_week_completed": last_week_completed,
                    "delta_moves": delta_moves,
                    "notification_type": notification_type,
                }
            ),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GenerateTextResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def converse(self, *, user_question: str) -> ConverseResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "converse"),
            json=jsonable_encoder({"user_question": user_question}),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ConverseResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncChima:
    def __init__(self, *, environment: ChimaEnvironment = ChimaEnvironment.PRODUCTION, token: str):
        self._environment = environment
        self._token = token

    async def search(
        self,
        group_id: str,
        *,
        query: str,
        provider: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        pdf: typing.Optional[str] = OMIT,
    ) -> SearchResponse:
        _request: typing.Dict[str, typing.Any] = {"query": query}
        if provider is not OMIT:
            _request["provider"] = provider
        if id is not OMIT:
            _request["id"] = id
        if pdf is not OMIT:
            _request["pdf"] = pdf
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", f"search/{group_id}"),
                json=jsonable_encoder(_request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SearchResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def stream(self, *, level_context: str) -> typing.AsyncIterator[str]:
        async with httpx.AsyncClient() as _client:
            async with _client.stream(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "api_streaming"),
                json=jsonable_encoder({"level_context": level_context}),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            ) as _response:
                if 200 <= _response.status_code < 300:
                    async for _text in _response.aiter_text():
                        if len(_text) == 0:
                            continue
                        yield pydantic.parse_obj_as(str, json.loads(_text))  # type: ignore
                    return
                try:
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(status_code=_response.status_code, body=_response.text)
                raise ApiError(status_code=_response.status_code, body=_response_json)

    async def generate(
        self,
        *,
        gender: str,
        country: str,
        age_bin: str,
        bmi_bin: str,
        active_journey: str,
        need: str,
        equipment: str,
        day_type: str,
        cluster_workout: str,
        cluster_challenge: str,
        last_workout_completed: str,
        last_workout_week: str,
        last_week_completed: int,
        delta_moves: str,
        notification_type: str,
    ) -> GenerateTextResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "generate"),
                json=jsonable_encoder(
                    {
                        "gender": gender,
                        "country": country,
                        "age_bin": age_bin,
                        "bmi_bin": bmi_bin,
                        "active_journey": active_journey,
                        "need": need,
                        "equipment": equipment,
                        "day_type": day_type,
                        "cluster_workout": cluster_workout,
                        "cluster_challenge": cluster_challenge,
                        "last_workout_completed": last_workout_completed,
                        "last_workout_week": last_workout_week,
                        "last_week_completed": last_week_completed,
                        "delta_moves": delta_moves,
                        "notification_type": notification_type,
                    }
                ),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GenerateTextResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def converse(self, *, user_question: str) -> ConverseResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "converse"),
                json=jsonable_encoder({"user_question": user_question}),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ConverseResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)
